@@ -79,7 +79,7 @@ impl ObjectImpl for VideoFallbackSource {
                     "URI",
                     "URI to use for video in case the main stream doesn't work",
                     None,
-                    glib::ParamFlags::READWRITE | glib::ParamFlags::CONSTRUCT_ONLY,
+                    glib::ParamFlags::READWRITE,
                 ),
                 glib::ParamSpecUInt64::new(
                     "min-latency",
@@ -88,7 +88,7 @@ impl ObjectImpl for VideoFallbackSource {
                     0,
                     std::u64::MAX,
                     0,
-                    glib::ParamFlags::READWRITE | glib::ParamFlags::CONSTRUCT_ONLY,
+                    glib::ParamFlags::READWRITE,
                 ),
             ]
         });
