@@ -20,7 +20,7 @@ use std::sync::{Mutex, RwLock};
 #[derive(Debug, Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy, glib::Enum)]
 #[repr(u32)]
 #[enum_type(name = "GstFallbackSwitchStreamHealth")]
-pub(crate) enum StreamHealth {
+pub enum StreamHealth {
     #[enum_value(name = "Data flow is inactive or late", nick = "inactive")]
     Inactive = 0,
     #[enum_value(name = "Data is currently flowing in the stream", nick = "present")]
@@ -212,30 +212,25 @@ impl FallbackSwitch {
     }
 
     #[allow(clippy::too_many_arguments)]
-    fn handle_main_buffer(
+    fn handle_main_timed_item(
         &self,
         agg: &super::FallbackSwitch,
         state: &mut OutputState,
         settings: &Settings,
-        mut buffer: gst::Buffer,
+        mut item: TimedItem,
         preferred_pad: &gst_base::AggregatorPad,
         backup_pad: &Option<&gst_base::AggregatorPad>,
         cur_running_time: impl Into<Option<gst::ClockTime>>,
-    ) -> Result<Option<(gst::Buffer, gst::Caps, bool)>, gst::FlowError> {
-        // If we got a buffer on the sinkpad just handle it
+    ) -> Result<Option<(TimedItem, gst::Caps, bool)>, gst::FlowError> {
         gst_debug!(
             CAT,
             obj: preferred_pad,
-            "Got buffer on pad {} - {:?}",
+            "Got {} on pad {} - {:?}",
+            item.name(),
             preferred_pad.name(),
-            buffer
+            item
         );
 
-        if buffer.pts().is_none() {
-            gst_error!(CAT, obj: preferred_pad, "Only buffers with PTS supported");
-            return Err(gst::FlowError::Error);
-        }
-
         let segment = preferred_pad
             .segment()
             .downcast::<gst::ClockTime>()
@@ -244,14 +239,27 @@ impl FallbackSwitch {
                 gst::FlowError::Error
             })?;
 
-        let running_time = segment.to_running_time(buffer.dts_or_pts());
+        let running_time = if item.pts().is_none() {
+            // re-use ts from previous buffer
+            let running_time = state
+                .primary
+                .last_sinkpad_time
+                .max(state.fallback.last_sinkpad_time);
 
-        {
-            // FIXME: This will not work correctly for negative DTS
-            let buffer = buffer.make_mut();
-            buffer.set_pts(segment.to_running_time(buffer.pts()));
-            buffer.set_dts(segment.to_running_time(buffer.dts()));
-        }
+            gst_debug!(
+                CAT,
+                obj: preferred_pad,
+                "{} does not have PTS, re-use ts from previous buffer: {}",
+                item.name(),
+                running_time.display()
+            );
+
+            running_time
+        } else {
+            segment.to_running_time(item.pts())
+        };
+
+        item.update_ts(running_time, &segment);
 
         if preferred_pad == &self.primary_sinkpad {
             state.primary.last_sinkpad_time = running_time;
@@ -272,7 +280,8 @@ impl FallbackSwitch {
             gst_debug!(
                 CAT,
                 obj: preferred_pad,
-                "Buffer is too late: {} > {}",
+                "{} is too late: {} > {}",
+                item.name(),
                 cur_running_time.display(),
                 deadline.display(),
             );
@@ -288,7 +297,8 @@ impl FallbackSwitch {
                 gst_debug!(
                     CAT,
                     obj: preferred_pad,
-                    "Buffer is too late and timeout reached: {} + {} <= {}",
+                    "{} is too late and timeout reached: {} + {} <= {}",
+                    item.name(),
                     state.last_output_time.display(),
                     settings.timeout,
                     running_time.display(),
@@ -303,7 +313,7 @@ impl FallbackSwitch {
             && active_sinkpad.as_ref() != Some(preferred_pad.upcast_ref::<gst::Pad>());
 
         if pad_change {
-            if buffer.flags().contains(gst::BufferFlags::DELTA_UNIT) {
+            if !item.is_keyframe() {
                 gst_info!(
                     CAT,
                     obj: preferred_pad,
@@ -340,7 +350,32 @@ impl FallbackSwitch {
             self.drain_pad_to_time(state, backup_pad, state.last_output_time)?;
         }
 
-        Ok(Some((buffer, active_caps, pad_change)))
+        Ok(Some((item, active_caps, pad_change)))
+    }
+
+    #[allow(clippy::too_many_arguments)]
+    fn handle_main_buffer(
+        &self,
+        agg: &super::FallbackSwitch,
+        state: &mut OutputState,
+        settings: &Settings,
+        buffer: gst::Buffer,
+        preferred_pad: &gst_base::AggregatorPad,
+        backup_pad: &Option<&gst_base::AggregatorPad>,
+        cur_running_time: impl Into<Option<gst::ClockTime>>,
+    ) -> Result<Option<(gst::Buffer, gst::Caps, bool)>, gst::FlowError> {
+        // If we got a buffer on the sinkpad just handle it
+        let res = self.handle_main_timed_item(
+            agg,
+            state,
+            settings,
+            TimedItem::Buffer(buffer),
+            preferred_pad,
+            backup_pad,
+            cur_running_time,
+        )?;
+
+        Ok(res.map(|res| (res.0.buffer(), res.1, res.2)))
     }
 
     fn backup_buffer(
@@ -363,11 +398,6 @@ impl FallbackSwitch {
                 buffer
             );
 
-            if buffer.pts().is_none() {
-                gst_error!(CAT, obj: backup_pad, "Only buffers with PTS supported");
-                return Err(gst::FlowError::Error);
-            }
-
             let backup_segment =
                 backup_pad
                     .segment()
@@ -376,12 +406,30 @@ impl FallbackSwitch {
                         gst_error!(CAT, obj: backup_pad, "Only TIME segments supported");
                         gst::FlowError::Error
                     })?;
-            let running_time = backup_segment.to_running_time(buffer.dts_or_pts());
+
+            let running_time = if buffer.pts().is_none() {
+                // re-use ts from previous buffer
+                let running_time = state
+                    .primary
+                    .last_sinkpad_time
+                    .max(state.fallback.last_sinkpad_time);
+
+                gst_debug!(
+                    CAT,
+                    obj: backup_pad,
+                    "Buffer does not have PTS, re-use ts from previous buffer: {}",
+                    running_time.display()
+                );
+
+                running_time
+            } else {
+                backup_segment.to_running_time(buffer.pts())
+            };
 
             {
                 // FIXME: This will not work correctly for negative DTS
                 let buffer = buffer.make_mut();
-                buffer.set_pts(backup_segment.to_running_time(buffer.pts()));
+                buffer.set_pts(running_time);
                 buffer.set_dts(backup_segment.to_running_time(buffer.dts()));
             }
 
@@ -951,7 +999,57 @@ impl AggregatorImpl for FallbackSwitch {
         match event.view() {
             EventView::Gap(gap) => {
                 if gap.gap_flags().contains(gst::GapFlags::DATA) {
-                    gst_debug!(CAT, obj: agg_pad, "Dropping gap event");
+                    let settings = self.settings.lock().unwrap().clone();
+                    let mut state = self.output_state.lock().unwrap();
+                    let fallback_sinkpad = self.fallback_sinkpad.read().unwrap();
+
+                    let on_active_sinkpad = {
+                        let active_sinkpad = self.active_sinkpad.lock().unwrap();
+                        active_sinkpad.as_ref() == Some(agg_pad.upcast_ref::<gst::Pad>())
+                    };
+
+                    // handle GAP as timed item
+                    let clock = agg.clock();
+                    let base_time = agg.base_time();
+
+                    let cur_running_time = if let Some(clock) = clock {
+                        clock.time().opt_checked_sub(base_time).ok().flatten()
+                    } else {
+                        gst::ClockTime::NONE
+                    };
+
+                    let mut forwarded = false;
+                    if let Ok(Some((item, _, _))) = self.handle_main_timed_item(
+                        agg,
+                        &mut *state,
+                        &settings,
+                        TimedItem::GapEvent(event),
+                        agg_pad,
+                        &fallback_sinkpad.as_ref(),
+                        cur_running_time,
+                    ) {
+                        // push GAP downstream only if it's from the active sink pad
+                        if on_active_sinkpad {
+                            let event = item.event();
+                            // FIXME: API to retrieve src pad from Aggregator?
+                            let src_pad = agg.static_pad("src").unwrap();
+
+                            gst_debug!(
+                                CAT,
+                                obj: agg_pad,
+                                "Forwarding gap event downstream: {:?}",
+                                event
+                            );
+
+                            src_pad.push_event(event);
+                            forwarded = true;
+                        }
+                    }
+
+                    if !forwarded {
+                        gst_debug!(CAT, obj: agg_pad, "Dropping gap event");
+                    }
+
                     Ok(gst::FlowSuccess::Ok)
                 } else {
                     self.parent_sink_event_pre_queue(agg, agg_pad, event)
@@ -1049,12 +1147,6 @@ impl AggregatorImpl for FallbackSwitch {
             .as_ref()
             .and_then(|p| p.peek_buffer().map(|buffer| (buffer, p)))
         {
-            if buffer.pts().is_none() {
-                gst_error!(CAT, obj: agg, "Only buffers with PTS supported");
-                // Trigger aggregate immediately to error out immediately
-                return Some(gst::ClockTime::ZERO);
-            }
-
             let segment = match backup_sinkpad.segment().downcast::<gst::ClockTime>() {
                 Ok(segment) => segment,
                 Err(_) => {
@@ -1064,7 +1156,26 @@ impl AggregatorImpl for FallbackSwitch {
                 }
             };
 
-            let running_time = segment.to_running_time(buffer.dts_or_pts());
+            let running_time = if buffer.pts().is_none() {
+                // re-use ts from previous buffer
+                let state = self.output_state.lock().unwrap();
+                let running_time = state
+                    .primary
+                    .last_sinkpad_time
+                    .max(state.fallback.last_sinkpad_time);
+
+                gst_debug!(
+                    CAT,
+                    obj: agg,
+                    "Buffer does not have PTS, re-use ts from previous buffer: {}",
+                    running_time.display(),
+                );
+
+                running_time
+            } else {
+                segment.to_running_time(buffer.pts())
+            };
+
             gst_debug!(
                 CAT,
                 obj: agg,
@@ -1097,7 +1208,7 @@ impl AggregatorImpl for FallbackSwitch {
 
         let pts = buffer.pts();
         if pts.is_none() {
-            gst_error!(CAT, obj: agg, "Only buffers with PTS supported");
+            gst_debug!(CAT, obj: agg, "Only clipping buffers with PTS supported");
             return Some(buffer);
         }
 
@@ -1224,3 +1335,87 @@ impl AggregatorImpl for FallbackSwitch {
         true
     }
 }
+
+#[derive(Debug)]
+enum TimedItem {
+    Buffer(gst::Buffer),
+    // only used with feature v1_20
+    #[allow(dead_code)]
+    GapEvent(gst::Event),
+}
+
+impl TimedItem {
+    fn name(&self) -> &str {
+        match self {
+            TimedItem::Buffer(_) => "buffer",
+            TimedItem::GapEvent(_) => "GAP event",
+        }
+    }
+
+    fn pts(&self) -> Option<gst::ClockTime> {
+        match self {
+            TimedItem::Buffer(buffer) => buffer.pts(),
+            TimedItem::GapEvent(event) => match event.view() {
+                gst::EventView::Gap(gap) => {
+                    let (pts, _duration) = gap.get();
+                    Some(pts)
+                }
+                _ => unreachable!(),
+            },
+        }
+    }
+
+    fn update_ts(
+        &mut self,
+        running_time: Option<gst::ClockTime>,
+        segment: &gst::FormattedSegment<gst::ClockTime>,
+    ) {
+        match self {
+            TimedItem::Buffer(buffer) => {
+                // FIXME: This will not work correctly for negative DTS
+                let buffer = buffer.make_mut();
+                buffer.set_pts(running_time);
+                buffer.set_dts(segment.to_running_time(buffer.dts()));
+            }
+            TimedItem::GapEvent(event) => {
+                let new_event = match event.view() {
+                    gst::EventView::Gap(gap) => {
+                        let (pts, duration) = gap.get();
+                        let builder = gst::event::Gap::builder(pts)
+                            .duration(duration)
+                            .seqnum(event.seqnum());
+
+                        #[cfg(feature = "v1_20")]
+                        let builder = builder.gap_flags(gap.gap_flags());
+
+                        builder.build()
+                    }
+                    _ => unreachable!(),
+                };
+                *event = new_event;
+            }
+        }
+    }
+
+    fn is_keyframe(&self) -> bool {
+        match self {
+            TimedItem::Buffer(buffer) => !buffer.flags().contains(gst::BufferFlags::DELTA_UNIT),
+            TimedItem::GapEvent(_) => false,
+        }
+    }
+
+    fn buffer(self) -> gst::Buffer {
+        match self {
+            TimedItem::Buffer(buffer) => buffer,
+            _ => unreachable!(),
+        }
+    }
+
+    #[cfg(feature = "v1_20")]
+    fn event(self) -> gst::Event {
+        match self {
+            TimedItem::GapEvent(event) => event,
+            _ => unreachable!(),
+        }
+    }
+}
